% A-optimal design for Example 5 (Atashgah and Seifi, (2007,2009))
% Two-response model with three independent variables
% One is a cubic model (8 terms), the other is quadratic model (6 terms)
% Design space consist of 19 vectors (x1,x2,x3)
% Use SeDuMi to solve problem
clear;
runningtime=cputime;
% design space 19X3 matrix, x1 x2 x3, 19 test vectors
u=[1.68 0 0; 0 1.68 0; 0 0 0; 1.729 1.727 -1.703; 1.728 -1.729 -1.72;
              1.729 1.729 1.729; -1.725 -1.723 1.715; -1.73 1.721 1.729;
              1.73 -1.729 1.729; -1.73 1.73 0.026; 1.73 -1.73 -0.045;
              -1.729 -1.73 -1.728; -1.73 -0.096 1.73; 1.729 1.724 -1.729;
              -0.154 1.73 -1.73; -0.101 -1.73 1.73; 1.729 1.729 1.722;
              -1.5168 -1.6182 0.652; 0.1158 1.6289 1.5256];
N=19;
r=0.1;  
sigma=[1 r;r 1]; % covariance matrix sigma0
invs=inv(sigma);

% r=2 is the number of response models
% q=q1+q2=8+6=14 is the number of parameters in the response model totally
U=zeros(2,14,N);
G=zeros(14,14,N);
% construct Uj j=1,...,19.
for i=1:N
    U(1,1:8,i)=[1 u(i,1) u(i,2) u(i,3) u(i,1)*u(i,2) u(i,1)*u(i,3) u(i,1)^2 u(i,3)^2];
    U(2,9:14,i)=[1 u(i,1) u(i,2) u(i,1)*u(i,2) u(i,1)^2 u(i,2)^2];
    G(:,:,i)=U(:,:,i)'*invs*U(:,:,i);
end

% each Hi is a [q*(q+1)+N]*[q*(q+1)+N] matrix
H=zeros(15*14+N,15*14+N,N+14);

for i=1:14
    H((i-1)*15+1:(i-1)*15+14,(i-1)*15+1:(i-1)*15+14,1)=G(:,:,N);
    H(16*i-15,15*i,1)=1;
    H(15*i,16*i-15,1)=1;
end
H(15*14+N,15*14+N,1)=1;

for j=2:N
    for i=1:14
    H((i-1)*15+1:(i-1)*15+14,(i-1)*15+1:(i-1)*15+14,j)=G(:,:,j-1)-G(:,:,N);
    end
    H(15*14+N,15*14+N,j)=-1;
    H(15*14-1+j,15*14-1+j,j)=1;
end

for i=1:14
    H(15*i,15*i,N+i)=1;
end

%target function is 0*w1+...+0*w(N-1)+1*v1+...+1*v14
c=zeros(N+13,1);
for i=N:N+13
    c(i)=1;
end

%SeDuMi
bt=-c;
ct=vec(H(:,:,1));
for i=1:N+13
    At(:,i)=-vec(H(:,:,i+1));
end
K.s=size(H(:,:,1),1);
pars.eps=0;
[x,y,info]=sedumi(At,bt,ct,K,pars);
info;
answer=zeros(N,1);
answer(1:N-1,1)=y(1:N-1,1)';
answer(N)=1-sum(y(1:N-1));

design=[u answer]

%check for optimality
z=zeros(14,14,N);
A=zeros(14,14);
for i=1:N
    z(:,:,i)=answer(i)*G(:,:,i);
    A=A+z(:,:,i);
end

phi=trace(inv(A));

check=zeros(N,1);
phi_A=zeros(N,1);
for i=1:N
    phi_A(i)=trace(A\G(:,:,i)/A);
    check(i)=phi_A(i)-phi;
end
delta=max(check)

